@@ -33,7 +33,7 @@ struct Backend
 };
 
 static std::atomic<int> g_receivedAnswers;
-static void report(std::unique_ptr<DNSPacket>& A)
+static void report(std::unique_ptr<DNSPacket>& A, int B)
 {
   g_receivedAnswers++;
 }
@@ -67,7 +67,7 @@ struct BackendSlow
 };
 
 static std::atomic<int> g_receivedAnswers1;
-static void report1(std::unique_ptr<DNSPacket>& A)
+static void report1(std::unique_ptr<DNSPacket>& A, int B)
 {
   g_receivedAnswers1++;
 }
@@ -119,7 +119,7 @@ std::atomic<int> BackendDies::s_count;
 
 std::atomic<int> g_receivedAnswers2;
 
-static void report2(std::unique_ptr<DNSPacket>& A)
+static void report2(std::unique_ptr<DNSPacket>& A, int B)
 {
   g_receivedAnswers2++;
 }
